﻿Imports System.Security.Cryptography

Public Class Main

#Region " Declarations "
  Private Const _WRONG_NUM_ARGUMENTS As Integer = 1
  Private Const _UNKNOWN_EXCEPTION As Integer = 2
  Private Const _VALIDATION_FAILED As Integer = 3

  Private Const _MAX_ARGUMENTS As Integer = 2
#End Region

#Region " Public Methods "
  Public Shared Sub Main(ByVal args() As String)
    If args.Length = 0 OrElse args.Length > _MAX_ARGUMENTS Then
      'Show usage and get out
      _DisplayUsage(_WRONG_NUM_ARGUMENTS, "Invalid number of arguments!")
      Exit Sub
    End If

    Try
      'Get the header data from the file
      Console.WriteLine("Reading file header...")
      Dim file As New IO.FileStream(args(0), IO.FileMode.Open)
      Dim headerData(&H4D) As Byte
      file.Read(headerData, 0, &H4E)

      'Determine the length of the OS data (including header)
      Dim l(3) As Byte
      file.Seek(&H50, IO.SeekOrigin.Begin)
      file.Read(l, 0, 4)
      'Convert Big Endian byte array to integer
      Dim length As Integer = (l(0) * &H1000000) + (l(1) * &H10000) + (l(2) * &H100) + l(3) + 6
      Console.WriteLine(String.Format("OS data length is 0x{0}...", length.ToString("X8")))

      'Read the OS data into our array
      Console.WriteLine("Reading OS data...")
      Dim data(Convert.ToInt32(length - 1)) As Byte
      file.Seek(&H4E, IO.SeekOrigin.Begin)
      file.Read(data, 0, length)
      file.Close()

      'Calculate the MD5 hash
      Console.WriteLine("Hashing OS data...")
      Dim md5 As New MD5CryptoServiceProvider()
      Dim hash() As Byte = md5.ComputeHash(data, 0, length)

      'Sign the hash
      Dim sig As New Signature(hash)

      If args.Length > 1 Then
        'Get D and N from the key file instead
        Console.WriteLine(String.Format("Key file {0} specified, reading content...", args(1)))
        Dim nums() As String = IO.File.ReadAllLines(args(1))
        Dim n As String = nums(0).Substring(2)
        Dim p As String = nums(1).Substring(2)
        Dim q As String = nums(2).Substring(2)

        'Reverse endianness of all strings
        n = _ReverseEndianness(n)
        p = _ReverseEndianness(p)
        q = _ReverseEndianness(q)

        sig.D = Signature.GetPrivateKeyExponent(p, q)
        sig.N = n
      Else
        Console.WriteLine("Key file not specified, defaulting to 0005...")
      End If

      If sig.Validate() Then
        'TODO: It's valid, so output the validation data (ew)
        Dim str As String = "020D"
        Dim tmp As String = sig.ToString()
        Dim ret(Convert.ToInt32(tmp.Length / 2) - 1) As Byte

        For i As Integer = ret.Length - 1 To 0 Step -1
          ret(i) = Convert.ToByte(tmp.Substring(0, 2), 16)
          tmp = tmp.Remove(0, 2)
        Next

        'Write the new file
        Console.WriteLine(String.Format("Re-writing file {0}...", args(0)))
        file = New IO.FileStream(args(0), IO.FileMode.Create)
        file.Write(headerData, 0, headerData.Length)
        file.Write(data, 0, data.Length)
        file.WriteByte(&H2)
        file.WriteByte(&HD)
        file.WriteByte(Convert.ToByte(ret.Length))
        For i As Integer = 0 To ret.Length - 1
          file.WriteByte(ret(i))
        Next
        file.Close()

        Console.WriteLine("Done.")
      Else
        Throw New SmartException(_VALIDATION_FAILED, "Generated signature did not pass validation!")
      End If
    Catch ex As SmartException
      _DisplayUsage(ex.ExitCode, ex.Message)
      Exit Sub
    Catch ex As Exception
      _DisplayUsage(_UNKNOWN_EXCEPTION, ex.ToString())
      Exit Sub
    End Try
  End Sub
#End Region

#Region " Local Methods "
  Private Shared Sub _DisplayUsage(ByVal exitCode As Integer, ByVal msg As String)
    Console.WriteLine(My.Application.Info.ProductName & " " & My.Application.Info.Version.ToString(3))
    Console.WriteLine("--------------------------------------------------")
    Console.WriteLine("Usage: resign68k <**K or **U file> [key file]")
    Console.WriteLine("Re-signs a 68k Flash application or OS upgrade file with the specified key file.")
    Console.WriteLine("The default key is 0005 if the key file is omitted.")
    Console.WriteLine()
    Console.WriteLine("Example: resign68k myapp.89k")
    Console.WriteLine("         resign68k OS.v2u 08.key")
    Console.WriteLine()
    Console.WriteLine("This application is horribly-written; please forgive me.")
    Console.WriteLine()
    Console.WriteLine("ERROR: " & msg)

    Environment.ExitCode = exitCode
  End Sub

  Private Shared Function _ReverseEndianness(ByVal s As String) As String
    Dim ret As String = String.Empty

    For i As Integer = 0 To Convert.ToInt32(s.Length / 2) - 1
      ret = s.Substring(0, 2) & ret
      s = s.Remove(0, 2)
    Next

    Return ret
  End Function
#End Region

End Class
